from lunespy.client.transactions.transfer_token.validators import validate_transfer
from lunespy.client.transactions.transfer_token.validators import mount_transfer
from lunespy.client.transactions.transfer_token.validators import send_transfer
from lunespy.client.transactions import BaseTransaction
from lunespy.client.wallet import Account
from lunespy.utils.settings import bcolors


class TransferToken(BaseTransaction):
    """
    transfer_data: dict
        @params fee_asset: int
        @params timestamp: int
        @params asset_id: str
        @params tx_fee: int
        @params amount: int
    """
    def __init__(self, sender: Account, receiver: Account, **transfer_data) -> None:
        self.sender: Account = sender
        self.receiver: Account = receiver
        self.transfer_data: dict = transfer_data
        self.history: list = []

    @property
    def ready(self) -> bool:
        return validate_transfer(
            self.sender,
            self.transfer_data
        )

    @property
    def transaction(self) -> dict:
        if self.ready:
            mount_tx = {'ready': True}
            mount_tx.update(mount_transfer(
                self.sender,
                self.receiver,
                self.transfer_data
                )
            )
            return mount_tx
        else:
            print(bcolors.FAIL + 'Transfer Transactions bad formed', bcolors.ENDC)
            return {'ready': False}
    @property
    def send(self) -> dict:
        mounted_tx = self.transaction
        if mounted_tx['ready']:
            tx_history = send_transfer(mounted_tx)
            self.history.append(tx_history)
            return tx_history
        else:
            print(bcolors.FAIL + 'Transfer Transactions dont send', bcolors.ENDC)
            return mounted_tx



from lunespy.client.transactions.issue.constants import DEFAULT_ISSUE_FEE
from lunespy.client.transactions.issue.constants import BYTE_TYPE_ISSUE
from lunespy.client.transactions.issue.constants import INT_TYPE_ISSUE
from lunespy.utils.crypto.converters import string_to_bytes
from lunespy.utils.crypto.converters import sign
from lunespy.utils.settings import bcolors
from lunespy.client.wallet import Account
from datetime import datetime
from base58 import b58decode
from requests import post
import struct


def mount_issue(creator: Account, issue_data: dict) -> dict:
    timestamp: int = issue_data.get('timestamp', int(datetime.now().timestamp() * 1000))
    issue_fee: int = issue_data.get('issue_fee', DEFAULT_ISSUE_FEE)
    reissuable: bool = issue_data.get('reissuable', False)
    description: str = issue_data.get('description', '')
    quantity: int = issue_data.get('quantity', 0)
    decimals: str = issue_data.get('decimals', 0)
    name: str = issue_data.get('name', '')

    bytes_data: bytes = BYTE_TYPE_ISSUE + \
        b58decode(creator.public_key) + \
        struct.pack(">H", len(name)) + \
        string_to_bytes(name) + \
        struct.pack(">H", len(description)) + \
        string_to_bytes(description) + \
        struct.pack(">Q", quantity) + \
        struct.pack(">B", decimals) + \
        (b'\1' if reissuable else b'\0') + \
        struct.pack(">Q", issue_fee) + \
        struct.pack(">Q", timestamp)

    signature: bytes = sign(creator.private_key, bytes_data)
    mount_tx = {
        "senderPublicKey": creator.public_key,
        "signature": signature.decode(),
        "description": description,
        "reissuable": reissuable,
        "timestamp": timestamp,
        "type": INT_TYPE_ISSUE,
        "decimals": decimals,
        "quantity": quantity,
        "fee": issue_fee,
        "name": name
    }
    return mount_tx


def validate_issue(creator: Account, issue_data: dict) -> bool:
    quantity: int = issue_data.get('quantity', -1)
    name: str = issue_data.get('name', '')

    if not creator.private_key:
        print(bcolors.FAIL + 'Sender `Account` not have a private key' + bcolors.ENDC)
        return False

    if quantity < 0:
        print(bcolors.FAIL + 'Issue_data `quantity` cannot be less than 0' + bcolors.ENDC)
        return False

    if len(name) not in range(4, 16 + 1):
        print(bcolors.FAIL + 'Asset name must be between 4 and 16 characters long' + bcolors.ENDC)
        return False

    return True


# todo async
def send_issue(mount_tx: dict, node: str) -> dict:
    response = post(
        f'{node}/transactions/broadcast',
        json=mount_tx,
        headers={
            'content-type':
            'application/json'
        })

    if response.ok:
        mount_tx['send'] = True
        mount_tx['response'] = response.json()
        return mount_tx
    else:
        mount_tx['send'] = False
        mount_tx['response'] = response.json()
        return mount_tx



from lunespy.server import NODE_URL
from requests import get


def aliases(address: str) -> str:
    pass


def balance(address: str) -> int:
    response = get(f'{NODE_URL}/addresses/balance/{address}')
    if response.ok:
        return response.json()['balance']
    else:
        return -1



from lunespy.client.transactions.reissue.constants import DEFAULT_REISSUE_FEE
from lunespy.client.transactions.reissue.constants import BYTE_TYPE_REISSUE
from lunespy.client.transactions.reissue.constants import INT_TYPE_REISSUE
from lunespy.utils.crypto.converters import sign
from lunespy.utils.settings import bcolors
from lunespy.client.wallet import Account
from datetime import datetime
from base58 import b58decode
from requests import post
import struct

def validate_reissue(creator: Account, reissue_data: dict) -> bool:
    quantity: int = reissue_data.get('quantity', -1)
    asset_id: str = reissue_data.get('asset_id', False)

    if not creator.private_key:
        print(bcolors.FAIL + 'Creator `Account` not have a private key' + bcolors.ENDC)
        return False

    if quantity < 0:
        print(bcolors.FAIL + 'Reissue_data `quantity` cannot be less than 0' + bcolors.ENDC)
        return False
    
    if asset_id == False:
        print(bcolors.FAIL + 'Reissue_data `asset_id` cannot exists' + bcolors.ENDC)
        return False

    return True


def mount_reissue(creator: Account, reissue_data: dict) -> dict:
    reissue_fee = reissue_data.get('reissue_fee', DEFAULT_REISSUE_FEE)
    timestamp = reissue_data.get('timestamp', int(datetime.now().timestamp() * 1000))
    reissuable = reissue_data.get('reissuable', False)
    asset_id = reissue_data['asset_id']
    quantity = reissue_data['quantity']

    bytes_data = BYTE_TYPE_REISSUE + \
        b58decode(creator.public_key) + \
        b58decode(asset_id) + \
        struct.pack(">Q", quantity) + \
        (b'\1' if reissuable else b'\0') + \
        struct.pack(">Q",reissue_fee) + \
        struct.pack(">Q", timestamp)

    signature: bytes = sign(creator.private_key, bytes_data)
    mount_tx = {
        "senderPublicKey": creator.public_key,
        "assetId": asset_id,
        "type": INT_TYPE_REISSUE,
        "reissuable": reissuable,
        "signature": signature.decode(),
        "timestamp": timestamp,
        "quantity": quantity,
        "fee": reissue_fee,
    }
    return mount_tx


# todo async
def send_reissue(mount_tx: dict, node: str) -> dict:
    response = post(
        f'{node}/transactions/broadcast',
        json=mount_tx,
        headers={
            'content-type':
            'application/json'
        })

    if response.ok:
        mount_tx['send'] = True
        mount_tx['response'] = response.json()
        return mount_tx
    else:
        mount_tx['send'] = False
        mount_tx['response'] = response.json()
        return mount_tx



from lunespy.client.transactions import BaseTransaction
from lunespy.client.transactions.reissue.validators import validate_reissue
from lunespy.client.transactions.reissue.validators import mount_reissue
from lunespy.client.transactions.reissue.validators import send_reissue
from lunespy.utils.settings import bcolors
from lunespy.client.wallet import Account


class ReissueToken(BaseTransaction):
    """
    data_reissue: dict
        @params asset_id: str, 
        @params quantity: int, 
        @params reissuable: bool, 
        @params tx_fee: int
    """
    def __init__(self, creator: Account, **reissue_data: dict) -> None:
        self.creator = creator
        self.reissue_data = reissue_data
        self.reissue_data['token_type'] = 'Token'
        self.history = []

    @property
    def ready(self) -> bool:
        return validate_reissue(self.creator, self.reissue_data)
    

    @property
    def transaction(self) -> dict:
        if self.ready:
            mount_tx = {'ready': True}
            mount_tx.update(mount_reissue(self.creator, self.reissue_data))
            return mount_tx
        else:
            print(bcolors.FAIL + 'Reissue Transaction bad formed', bcolors.ENDC)
            mount_tx = {'ready': False}
            return mount_tx

    def send(self, http_node: str='') -> dict:
        mounted_tx = self.transaction
        if mounted_tx['ready']:
            node = http_node if http_node else NODE_URL
            tx_history = send_reissue(mounted_tx, node=node)
            self.history.append(tx_history)
            if tx_history['send']:
                self.successful(tx_history['response'], self.reissue_data['token_type'])
            else:
                print(bcolors.FAIL + f'Your {self.reissue_data["token_type"]} dont reissued because:\n', bcolors.ENDC)
                print(tx_history['response'])   
            return tx_history
        else:
            print(bcolors.FAIL + 'Reissue Transaction dont send', bcolors.ENDC)
            return mounted_tx

    def successful(self, asset_reissued: dict, token_type: str) -> None:
        reissuable = asset_reissued['reissuable']
        quantity = asset_reissued['quantity']
        transaction_id = asset_reissued['id']
        asset_id = asset_reissued['assetId']
        creator = asset_reissued['sender']
        asset_reissued.update({'token_type': token_type})

        print(f"\
            \ntype\n {bcolors.OKGREEN + '└──' +  token_type + bcolors.ENDC}\
            \nasset_id\n {bcolors.OKBLUE + '└──' + asset_id + bcolors.ENDC}\
            \nquantity\n {bcolors.OKBLUE + '└──' + str(quantity) + bcolors.ENDC}\
            \ncreator\n {bcolors.OKBLUE + '└──' + creator + bcolors.ENDC}\
            \nreissuable\n {bcolors.OKBLUE + '└──' + str(reissuable) + bcolors.ENDC}\
            \ntransaction_id\n {bcolors.OKBLUE + '└──' + transaction_id + bcolors.ENDC}\
        ")
        
        import json
        with open(f'./reissue-token.json', 'w') as file:
            file.write(json.dumps(asset_reissued))

        print(f"\n{bcolors.OKGREEN}Your {token_type} has been reissued and saved in `./reissuetoken.json`{bcolors.ENDC}")


class ReissueAsset(ReissueToken):
    def __init__(self,
        creator: Account,
        **reissue_data: dict
        ) -> None:
        reissue_data['token_type'] = reissue_data['token_type'] if reissue_data.get('token_type', False) else 'Asset'
        super().__init__(creator, **reissue_data)


class ReissueNFT(ReissueAsset):
    def __init__(self,
        creator: Account,
        **reissue_data: dict
        ) -> None:
        reissue_data['token_type'] = 'NFT'
        reissue_data['decimals'] = 0
        super().__init__(creator, **reissue_data)


from abc import ABCMeta, abstractmethod

class BaseTransaction(metaclass=ABCMeta):

    @abstractmethod
    def ready(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    def transaction(self) -> dict:
        raise NotImplementedError

    @abstractmethod
    def send(self) -> dict:
        raise NotImplementedError


from lunespy.client.transactions.transfer_token.constants import DEFAULT_TRANSFER_FEE
from lunespy.client.transactions.transfer_token.constants import BYTE_TYPE_TRANSFER
from lunespy.client.transactions.transfer_token.constants import INT_TYPE_TRANSFER
from lunespy.utils.crypto.converters import sign
from lunespy.utils.settings import bcolors
from lunespy.client.wallet import Account
from lunespy.server import NODE_URL
from datetime import datetime
from base58 import b58decode
from requests import post
import struct


def mount_transfer(sender: Account, receiver: Account, tx_data: dict) -> dict:
    timestamp: int = tx_data.get('timestamp', int(datetime.now().timestamp() * 1000))
    amount: int = tx_data['amount']
    transfer_fee: int = tx_data.get('transfer_fee', DEFAULT_TRANSFER_FEE)
    asset_id: str = tx_data.get('asset_id', "")
    fee_asset: str = tx_data.get('fee_asset', "")

    bytes_data: bytes = BYTE_TYPE_TRANSFER + \
        b58decode(sender.public_key) + \
        (b'\1' + b58decode(asset_id) if asset_id != "" else b'\0') + \
        (b'\1' + b58decode(fee_asset) if fee_asset != "" else b'\0') + \
        struct.pack(">Q", timestamp) + \
        struct.pack(">Q", amount) + \
        struct.pack(">Q", transfer_fee) + \
        b58decode(receiver.address)
    signature: bytes = sign(sender.private_key, bytes_data)
    mount_tx: dict = {
        "senderPublicKey": sender.public_key,
        "signature": signature.decode(),
        "timestamp": timestamp,
        "recipient": receiver.address,
        "feeAsset": fee_asset,
        "assetId": asset_id,
        "amount": amount,
        "type": INT_TYPE_TRANSFER,
        "fee": transfer_fee
    }
    return mount_tx


def validate_transfer(sender: Account, tx_data: dict) -> bool:
    amount: int = tx_data.get('amount', -1)

    if not sender.private_key:
        print(bcolors.FAIL + 'Sender `Account` not have a private key' + bcolors.ENDC)
        return False
    elif amount <= 0:
        print(bcolors.FAIL + 'You dont pass `amount` param' + bcolors.ENDC)
        return False
    return True


# todo async
def send_transfer(mount_tx: dict) -> dict:
    response = post(
        f'{NODE_URL}/transactions/broadcast',
        json=mount_tx,
        headers={
            'content-type':
            'application/json'
        })

    if response.ok:
        mount_tx['send'] = True
        mount_tx['response'] = response.json()
        return mount_tx
    else:
        mount_tx['send'] = False
        mount_tx['response'] = response.json()
        return mount_tx



from lunespy.utils.crypto.converters import bytes_to_string
from lunespy.utils.crypto.converters import string_to_bytes
from lunespy.utils.crypto.converters import hash_chain
from lunespy.client.wallet.constants import word_list
import axolotl_curve25519 as curve
from base58 import b58decode
from base58 import b58encode
from os import urandom



def address_generator(public_key: str, chain_id: str) -> dict:
    un_hashed_address = chr(1) + str(chain_id) + hash_chain(public_key)[0:20]
    address_hash = hash_chain(string_to_bytes(un_hashed_address))[0:4]
    address = b58encode(string_to_bytes(un_hashed_address + address_hash))
    public_key_b58 = b58encode(public_key)
    return {
        'seed': "",
        'hash_seed': "",
        'nonce': 0,
        'private_key': "",
        'public_key': public_key_b58.decode(),
        'address': address.decode(),
        'byte_private_key': b"",
        'byte_public_key': public_key_b58,
        'byte_address': address

    }


def new_seed_generator() -> str:
    wordCount = 2048
    words = []
    for i in range(5):
        r = bytes_to_string(urandom(4))
        x = (ord(r[3])) + (ord(r[2]) << 8) + (ord(r[1]) << 16) + (ord(r[0]) << 24)
        w1 = x % wordCount
        w2 = ((int(x / wordCount) >> 0) + w1) % wordCount
        w3 = ((int((int(x / wordCount) >> 0) / wordCount) >> 0) + w2) % wordCount
        words.append(word_list[w1])
        words.append(word_list[w2])
        words.append(word_list[w3])
    return " ".join(words)


def seed_generator(seed: str, nonce: int, chain_id: str) -> dict:
    from lunespy.utils.crypto.converters import sha256
    import struct
    
    hash_seed = hash_chain(struct.pack(">L", nonce) + string_to_bytes(seed))
    account_hash_seed = sha256(hash_seed)
    private_key = curve.generatePrivateKey(account_hash_seed)
    public_key = curve.generatePublicKey(private_key)
    address = address_generator(public_key, chain_id)
    return {
        'seed': seed,
        'hash_seed': b58encode(seed).decode(),
        'nonce': nonce,
        'private_key': b58encode(private_key).decode(),
        'public_key': b58encode(public_key).decode(),
        'address': address['address'],
        'byte_private_key': b58encode(private_key),
        'byte_public_key': b58encode(public_key),
        'byte_address': address['byte_address']
    }


def private_key_generator(private_key: str, chain_id: str) -> dict:
    private_key_b58 = b58decode(private_key)
    public_key = curve.generatePublicKey(private_key_b58)
    address = address_generator(public_key, chain_id)
    return {
        'seed': "",
        'hash_seed': "",
        'nonce': 0,
        'private_key': b58encode(private_key_b58).decode(),
        'public_key': b58encode(public_key).decode(),
        'address': address['address'],
        'byte_private_key': b58encode(private_key_b58),
        'byte_public_key': b58encode(public_key),
        'byte_address': address['byte_address']

    }
    

def public_key_generator(public_key: str, chain_id: str) -> dict:
    public_key_b58 = b58decode(public_key)
    address = address_generator(public_key_b58, chain_id)
    return {
        'seed': "",
        'hash_seed': "",
        'nonce': 0,
        'private_key': "",
        'public_key': b58encode(public_key_b58).decode(),
        'address': address['address'],
        'byte_private_key': b"",
        'byte_public_key': b58encode(public_key_b58),
        'byte_address': address['byte_address']
    }



def wallet_generator(**data: dict) -> dict:
    if data.get('seed', False):
        return seed_generator(seed=data['seed'], nonce=data['nonce'], chain_id=data['chain_id'])

    elif data.get('private_key', False):
        return private_key_generator(private_key=data['private_key'], chain_id=data['chain_id'])

    elif data.get('public_key', False):
        return public_key_generator(public_key=data['public_key'], chain_id=data['chain_id'])

    elif data.get('address', False):
        return address_generator(public_key="", chain_id=data['chain_id'])

    else:
        seed = new_seed_generator()
        return seed_generator(seed=seed, nonce=data['nonce'], chain_id=data['chain_id'])


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lunespy',
 'lunespy.client',
 'lunespy.client.transactions',
 'lunespy.client.transactions.issue',
 'lunespy.client.transactions.reissue',
 'lunespy.client.transactions.transfer_token',
 'lunespy.client.wallet',
 'lunespy.server',
 'lunespy.server.address',
 'lunespy.utils',
 'lunespy.utils.crypto',
 'lunespy.utils.settings']

package_data = \
{'': ['*']}

install_requires = \
['base58>=2.1.0,<3.0.0',
 'pyblake2>=1.1.2,<2.0.0',
 'python-axolotl-curve25519>=0.4.1.post2,<0.5.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'lunespy',
    'version': '1.3.0',
    'description': 'Library for communication with nodes in mainnet or testnet of the lunes-blockchain network',
    'long_description': None,
    'author': 'Lunes Platform',
    'author_email': 'lucas.oliveira@lunes.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from lunespy.client.transactions.issue.validators import validate_issue
from lunespy.client.transactions.issue.validators import send_issue
from lunespy.client.transactions.issue.validators import mount_issue
from lunespy.client.transactions import BaseTransaction
from lunespy.utils.settings import bcolors
from lunespy.client.wallet import Account
from lunespy.server import NODE_URL

class IssueToken(BaseTransaction):
    """
    issue_data: dict
        @params description: str
        @params reissuable: bool
        @params quantity: int
        @params decimals: int
        @params tx_fee: int
        @params name: str
    """
    def __init__(self, creator: Account, **issue_data: dict) -> None:
        self.creator = creator
        self.issue_data = issue_data
        self.issue_data['token_type'] = 'Token'
        self.history = []

    @property
    def ready(self) -> bool:
        return validate_issue(
            self.creator,
            self.issue_data
        )

    @property
    def transaction(self) -> dict:
        if self.ready:
            mount_tx = {'ready': True}
            mount_tx.update(mount_issue(self.creator, self.issue_data))
            return mount_tx
        else:
            print(bcolors.FAIL + 'Issue Transaction bad formed', bcolors.ENDC)
            mount_tx = {'ready': False}
            return mount_tx


    def send(self, http_node: str='') -> dict:
        mounted_tx = self.transaction
        if mounted_tx['ready']:
            node = http_node if http_node else NODE_URL
            tx_history = send_issue(mounted_tx, node=node)
            self.history.append(tx_history)
            if tx_history['send']:
                self.successful(tx_history['response'], self.issue_data['token_type'])
            else:
                print(bcolors.FAIL + f'Your {self.issue_data["token_type"]} dont issued because:\n', bcolors.ENDC)
                print(tx_history['response'])   
            return tx_history
        else:
            print(bcolors.FAIL + 'Issue Transaction dont send', bcolors.ENDC)
            return mounted_tx

    def successful(self, asset_issued: dict, token_type: str) -> None:
        description = asset_issued['description']
        reissuable = asset_issued['reissuable']
        quantity = asset_issued['quantity']
        decimals = asset_issued['decimals']
        transaction_id = asset_issued['id']
        asset_id = asset_issued['assetId']
        creator = asset_issued['sender']
        name = asset_issued['name']
        asset_issued.update({'token_type': token_type})

        print(f"\
            \nname\n {bcolors.OKGREEN + '└──' +  name + bcolors.ENDC}\
            \ntype\n {bcolors.OKGREEN + '└──' +  token_type + bcolors.ENDC}\
            \nasset_id\n {bcolors.OKBLUE + '└──' + asset_id + bcolors.ENDC}\
            \nquantity\n {bcolors.OKBLUE + '└──' + str(quantity) + bcolors.ENDC}\
            \ncreator\n {bcolors.OKBLUE + '└──' + creator + bcolors.ENDC}\
            \ndescription\n {bcolors.OKBLUE + '└──' + description + bcolors.ENDC}\
            \nreissuable\n {bcolors.OKBLUE + '└──' + str(reissuable) + bcolors.ENDC}\
            \ndecimals\n {bcolors.OKBLUE + '└──' + str(decimals) + bcolors.ENDC}\
            \ntransaction_id\n {bcolors.OKBLUE + '└──' + transaction_id + bcolors.ENDC}\
        ")
        
        import json
        with open(f'./token-{name}.json', 'w') as file:
            file.write(json.dumps(asset_issued))

        print(f"\n{bcolors.OKGREEN}Your {token_type} has been issued and saved in `./asset_info.json`{bcolors.ENDC}")


class IssueAsset(IssueToken):
    def __init__(self,
        creator: Account,
        **issue_data: dict
        ) -> None:
        issue_data['token_type'] = issue_data['token_type'] if issue_data.get('token_type', False) else 'Asset'
        super().__init__(creator, **issue_data)


class IssueNFT(IssueAsset):
    def __init__(self,
        creator: Account,
        **issue_data: dict
        ) -> None:
        issue_data['token_type'] = 'NFT'
        issue_data['decimals'] = 0
        super().__init__(creator, **issue_data)


from lunespy.client.wallet.validators import validate_wallet


class Account:
    def __init__(self, **wallet: dict):
        validated_wallet = validate_wallet(wallet)
        
        __validated_wallet = validate_wallet(wallet)
        
        self.private_key = __validated_wallet['private_key']
        self.public_key = __validated_wallet['public_key']
        self.hash_seed = __validated_wallet['hash_seed']
        self.address = __validated_wallet['address']
        self.nonce = __validated_wallet['nonce']
        self.seed = __validated_wallet['seed']

        self.byte_private_key = __validated_wallet['byte_private_key']
        self.byte_public_key = __validated_wallet['byte_public_key']
        self.byte_address = __validated_wallet['byte_address']

    def __str__(self) -> str:
        from lunespy.utils.settings import bcolors

        return f"\
            \nseed\n {bcolors.OKGREEN + '└── ' +  self.seed + bcolors.ENDC}\
            \nnonce\n {bcolors.OKBLUE + '└── ' + str(self.nonce) + bcolors.ENDC}\
            \nprivate key\n {bcolors.OKBLUE + '└── ' + self.private_key + bcolors.ENDC}\
            \npublic key\n {bcolors.OKBLUE + '└── ' + self.public_key + bcolors.ENDC}\
            \naddress\n {bcolors.OKBLUE + '└── ' + self.address + bcolors.ENDC}"

    __repr__ = __str__

    def to_json(self, path: str = './') -> None:
        from lunespy.utils.settings import bcolors
        import json

        wallet = {
            'seed': self.seed,
            'nonce': self.nonce,
            'private_key': self.private_key,
            'public_key': self.public_key,
            'address': self.address
        }
        with open(f'{path}wallet.json', 'w') as file:
            file.write(json.dumps(wallet))
        print(f"{bcolors.OKGREEN}Your wallet has been saved in `{path}wallet.json`{bcolors.ENDC}")